/*
 * Copyright (C) 2007 Brice Figureau
 * shm_zone and rbtree code Copyright (c) 2002-2007 Igor Sysoev
 */
/*
 * Copyright (C) 2009   Lev Alexandrovsky <lev0666@yandex.ru>, 
                        Alexey Guseynov <aguseynov@gmail.com>
 * integration with Nginx upload module (v 2.0.10)
 */

#include <ngx_config.h>
#include <ngx_core.h>
#include <ngx_http.h>

#define TIMER_FREQUENCY 15 * 1000



typedef struct ngx_http_uploadprogress_node_s ngx_http_uploadprogress_node_t;

struct ngx_http_uploadprogress_node_s {
    ngx_rbtree_node_t                node;
    ngx_uint_t                       err_status;
    off_t                            rest;
    off_t                            length;
    ngx_uint_t                       done;
    time_t                           timeout;
    struct ngx_http_uploadprogress_node_s *prev;
    struct ngx_http_uploadprogress_node_s *next;
    u_char                           len;
    u_char                           data[1];
};

typedef struct {
    ngx_shm_zone_t                  *shm_zone;
    ngx_rbtree_node_t               *node;
    ngx_http_request_t              *r;
    time_t                           timeout;
} ngx_http_uploadprogress_cleanup_t;

typedef struct {

    ngx_shm_zone_t                  *shm_zone;
    time_t                           timeout;
    ngx_event_t                      cleanup;
    ngx_http_handler_pt              handler;
} ngx_http_uploadprogress_conf_t;

typedef struct {
    ngx_str_t         url;
    ngx_path_t        *store_path;
    ngx_uint_t        store_access;
    size_t            buffer_size;
    size_t            max_header_len;
    size_t            max_output_body_len;
    off_t             max_file_size;
    ngx_array_t       *field_templates;
    ngx_array_t       *aggregate_field_templates;
    ngx_array_t       *field_filters;
    ngx_array_t       *cleanup_statuses;
    ngx_flag_t         forward_args;
    size_t            limit_rate;

    unsigned int      md5:1;
    unsigned int      sha1:1;unsigned int      crc32:1;
	
/* progress module */	
	u_char                          track;
	ngx_http_uploadprogress_conf_t 	*track_info;
	
} ngx_http_upload_loc_conf_t_fake; // for type conversion


typedef struct {
    ngx_rbtree_t                    *rbtree;
    ngx_http_uploadprogress_node_t   list_head;
    ngx_http_uploadprogress_node_t   list_tail;
	ngx_http_uploadprogress_node_t   *current_node;
} ngx_http_uploadprogress_ctx_t;



extern  ngx_int_t ngx_http_upload_progress_init(ngx_http_request_t * r, ngx_http_uploadprogress_conf_t *conf, u_char track);
extern void ngx_http_upload_progress_finish(ngx_http_request_t *r, ngx_http_uploadprogress_conf_t *track_info);
extern  char *track_upload_handler(ngx_http_uploadprogress_conf_t *track_info, u_char *track, ngx_conf_t * cf, ngx_module_t *ngx_http_upload_module);
extern  char *report_upload_handler(ngx_http_uploadprogress_conf_t *track_info, u_char *track, ngx_conf_t * cf, ngx_module_t *ngx_http_upload_module);
extern  ngx_int_t report_handler(ngx_http_request_t * r, ngx_http_uploadprogress_conf_t *track_info);
extern  char* upload_progress_handler(ngx_conf_t * cf, ngx_module_t *ngx_http_upload_module);
extern void change_progress(ngx_http_uploadprogress_conf_t *track_info, ngx_http_request_t *r);
